# Generated by Django 3.0.4 on 2020-03-26 15:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0050_deleted_progresses'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Mat_view',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-30 16:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0037_auto_20190822_2250'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Messages_theme_access',
        ),
    ]



# Generated by Django 2.2.5 on 2019-09-04 18:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0055_auto_20190904_1757'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='messages',
            unique_together=set(),
        ),
    ]



# Generated by Django 2.2.5 on 2019-09-19 16:20

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('twits', '0018_auto_20190916_1759'),
    ]

    operations = [
        migrations.AlterField(
            model_name='chat_user_user_theme',
            name='theme',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='tracker.Messages_theme'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-17 11:35

from django.db import migrations
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('history', '0002_history_username'),
    ]

    operations = [
        migrations.AddField(
            model_name='history',
            name='fio',
            field=isc_common.fields.name_field.NameField(blank=True, null=True),
        ),
    ]



from django.urls import path
from isc_common.auth.views.widgets_trees import Widgets_trees_Fetch, Widgets_trees_Add, Widgets_trees_Update, Widgets_trees_Remove, Widgets_trees_Info, Widgets_trees_Lookup

urlpatterns = [

    path('Widgets_trees/Fetch/', Widgets_trees_Fetch),
    path('Widgets_trees/Add', Widgets_trees_Add),
    path('Widgets_trees/Update', Widgets_trees_Update),
    path('Widgets_trees/Remove', Widgets_trees_Remove),
    path('Widgets_trees/Lookup/', Widgets_trees_Lookup),
    path('Widgets_trees/Info/', Widgets_trees_Info),

]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from isc_common.models.last_used_processes import Last_used_processes, Last_used_processesManager


@JsonResponseWithException()
def Last_used_processes_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Last_used_processes.objects.
                select_related().
                get_range_rows1(
                request=request,
                function=Last_used_processesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Last_used_processes_Add(request):
    return JsonResponse(DSResponseAdd(data=Last_used_processes.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Last_used_processes_Update(request):
    return JsonResponse(DSResponseUpdate(data=Last_used_processes.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Last_used_processes_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Last_used_processes.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Last_used_processes_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Last_used_processes.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Last_used_processes_Info(request):
    return JsonResponse(DSResponse(request=request, data=Last_used_processes.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Last_used_processes_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Last_used_processes.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.3 on 2019-07-03 16:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('refs', '0004_type_param_values'),
    ]

    operations = [
        migrations.RenameField(
            model_name='type_param',
            old_name='pick_list',
            new_name='value_map',
        ),
        migrations.RemoveField(
            model_name='type_param',
            name='exponent',
        ),
        migrations.RemoveField(
            model_name='type_param',
            name='precision',
        ),
        migrations.AddField(
            model_name='type_param',
            name='required',
            field=models.BooleanField(default=False),
        ),
    ]



import logging

from django.db.models import Model

from isc_common.fields.code_field import CodeField

logger = logging.getLogger(__name__)


class Param_type(Model):
    code = CodeField(unique=True, db_index=True)

    def __str__(self):
        return f"(id: {self.id}, code: {self.code})"

    class Meta:
        verbose_name = 'Типы параметров'



# Generated by Django 2.2.5 on 2019-09-03 19:00

from django.conf import settings
from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('tracker', '0043_messages_files'),
    ]

    operations = [
        migrations.AddField(
            model_name='messages',
            name='to_whom',
            field=isc_common.fields.related.ForeignKeyCascade(default=None, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-21 00:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0004_calendar_shits_days'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='day',
            name='begintime',
        ),
        migrations.RemoveField(
            model_name='day',
            name='endtime',
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-23 08:22

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('events', '0003_auto_20191118_1245'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event_type_users',
            name='event_type',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='events.Event_types'),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-17 09:59

import django.db.models.deletion
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('events', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event_type_users',
            name='event_type',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='events.Event_types'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-17 18:02

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('tracker', '0021_messages_whom'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='messages_whom',
            unique_together={('message', 'to_whom')},
        ),
    ]



import datetime
import logging

from django.core.management import BaseCommand
from django.db import connections, connection, transaction

from history.models.history import History

logger = logging.getLogger(__name__)


class Command(BaseCommand):

    def handle(self, *args, **options):
        with connections['history'].cursor() as cursor:
            begdate = datetime.datetime.today().replace(hour=0, minute=0, second=0, microsecond=0)
            begdate = begdate.replace(day=begdate.day - 1, hour=0, minute=0, second=0, microsecond=0)
            enddate = begdate.replace(day=begdate.day + 1)
            cursor.execute('''select
                                    username,
                                    fio,
                                    count(*)
                                from history_history
                                where date between %s and %s
                                group by username, fio
                                order by fio ''', [begdate, enddate])
            count_blocks = cursor.fetchall()
            res = [dict(username=count_block[0], fio=count_block[1], count=count_block[2]) for count_block in count_blocks]
            for r in res:
                print(r)

        print('Aggregate done.')



from typing import *

CSSColor = Text

WebSocketDataType = Union["message", "reload_browser", "refresh_chat_menu", "error", "refresh", "uploaded_file", "delivered", "readed"]


class ChatData:
    def __init__(self, kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v() if callable(v) else v)

    channel: Text = None
    date: Text = None
    guid: Optional[Text] = None
    host: Text = None
    message: Optional[Text] = None
    message_state_delivered_id: int = None
    message_state_delivered_name: Text = None
    message_state_new_id: int = None
    message_state_new_name: Text = None
    message_state_not_readed_id: int = None
    message_state_not_readed_name: Text = None
    message_state_readed_id: int = None
    message_state_readed_name: Text = None
    not_: int = None
    path: Text = None
    port: int = None
    props: int = None
    theme_ids: List[int] = None
    type: WebSocketDataType = None
    user__color: Text = None
    user__full_name: Text = None
    user__short_name: Text = None
    user_id: int = None


class ChatResponseData(Dict):
    channel: Text
    date: Text
    guid: Text
    host: Text
    message: Text
    port: Text
    state__name: Optional[Text]
    to_whom_id: Optional[int]
    type: WebSocketDataType
    user__color: CSSColor
    user__short_name: Text
    user_id: int



# Generated by Django 3.0.3 on 2020-02-08 14:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0016_remove_files_1c_id_old'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='documents_param_1c',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_boolean', None), ('value_float', None), ('value_int', None), ('value_uuid', None)), fields=('type',), name='Documents_param_1c_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value_boolean', None), ('value_float', None), ('value_int', None), ('value_uuid', None)), fields=('type', 'value'), name='Documents_param_1c_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_float', None), ('value_int', None), ('value_uuid', None)), fields=('type', 'value_boolean'), name='Documents_param_1c_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value_float', None), ('value_int', None), ('value_uuid', None)), fields=('type', 'value', 'value_boolean'), name='Documents_param_1c_unique_constraint_3'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_boolean', None), ('value_int', None), ('value_uuid', None)), fields=('type', 'value_float'), name='Documents_param_1c_unique_constraint_4'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value_boolean', None), ('value_int', None), ('value_uuid', None)), fields=('type', 'value', 'value_float'), name='Documents_param_1c_unique_constraint_5'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_int', None), ('value_uuid', None)), fields=('type', 'value_boolean', 'value_float'), name='Documents_param_1c_unique_constraint_6'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value_int', None), ('value_uuid', None)), fields=('type', 'value', 'value_boolean', 'value_float'), name='Documents_param_1c_unique_constraint_7'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_boolean', None), ('value_float', None), ('value_uuid', None)), fields=('type', 'value_int'), name='Documents_param_1c_unique_constraint_8'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value_boolean', None), ('value_float', None), ('value_uuid', None)), fields=('type', 'value', 'value_int'), name='Documents_param_1c_unique_constraint_9'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_float', None), ('value_uuid', None)), fields=('type', 'value_boolean', 'value_int'), name='Documents_param_1c_unique_constraint_10'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value_float', None), ('value_uuid', None)), fields=('type', 'value', 'value_boolean', 'value_int'), name='Documents_param_1c_unique_constraint_11'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_boolean', None), ('value_uuid', None)), fields=('type', 'value_float', 'value_int'), name='Documents_param_1c_unique_constraint_12'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value_boolean', None), ('value_uuid', None)), fields=('type', 'value', 'value_float', 'value_int'), name='Documents_param_1c_unique_constraint_13'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_uuid', None)), fields=('type', 'value_boolean', 'value_float', 'value_int'), name='Documents_param_1c_unique_constraint_14'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(value_uuid=None), fields=('type', 'value', 'value_boolean', 'value_float', 'value_int'), name='Documents_param_1c_unique_constraint_15'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_boolean', None), ('value_float', None), ('value_int', None)), fields=('type', 'value_uuid'), name='Documents_param_1c_unique_constraint_16'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value_boolean', None), ('value_float', None), ('value_int', None)), fields=('type', 'value', 'value_uuid'), name='Documents_param_1c_unique_constraint_17'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_float', None), ('value_int', None)), fields=('type', 'value_boolean', 'value_uuid'), name='Documents_param_1c_unique_constraint_18'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value_float', None), ('value_int', None)), fields=('type', 'value', 'value_boolean', 'value_uuid'), name='Documents_param_1c_unique_constraint_19'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_boolean', None), ('value_int', None)), fields=('type', 'value_float', 'value_uuid'), name='Documents_param_1c_unique_constraint_20'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value_boolean', None), ('value_int', None)), fields=('type', 'value', 'value_float', 'value_uuid'), name='Documents_param_1c_unique_constraint_21'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_int', None)), fields=('type', 'value_boolean', 'value_float', 'value_uuid'), name='Documents_param_1c_unique_constraint_22'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(value_int=None), fields=('type', 'value', 'value_boolean', 'value_float', 'value_uuid'), name='Documents_param_1c_unique_constraint_23'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_boolean', None), ('value_float', None)), fields=('type', 'value_int', 'value_uuid'), name='Documents_param_1c_unique_constraint_24'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value_boolean', None), ('value_float', None)), fields=('type', 'value', 'value_int', 'value_uuid'), name='Documents_param_1c_unique_constraint_25'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_float', None)), fields=('type', 'value_boolean', 'value_int', 'value_uuid'), name='Documents_param_1c_unique_constraint_26'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(value_float=None), fields=('type', 'value', 'value_boolean', 'value_int', 'value_uuid'), name='Documents_param_1c_unique_constraint_27'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(('value', None), ('value_boolean', None)), fields=('type', 'value_float', 'value_int', 'value_uuid'), name='Documents_param_1c_unique_constraint_28'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(value_boolean=None), fields=('type', 'value', 'value_float', 'value_int', 'value_uuid'), name='Documents_param_1c_unique_constraint_29'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(condition=models.Q(value=None), fields=('type', 'value_boolean', 'value_float', 'value_int', 'value_uuid'), name='Documents_param_1c_unique_constraint_30'),
        ),
        migrations.AddConstraint(
            model_name='documents_param_1c',
            constraint=models.UniqueConstraint(fields=('type', 'value', 'value_boolean', 'value_float', 'value_int', 'value_uuid'), name='Documents_param_1c_unique_constraint_31'),
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-06 21:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0013_auto_20190706_2007'),
    ]

    operations = [
        migrations.AlterField(
            model_name='uuid_2_id',
            name='uuid',
            field=models.UUIDField(db_index=True, unique=True),
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-03 17:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('refs', '0005_auto_20190703_1646'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='type_param_values',
            name='type',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-23 09:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0037_auto_20190823_0929'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='shift_day',
            unique_together=set(),
        ),
    ]



from isc_common import Wrapper


class Criterion(Wrapper):
    criteria = None
    end = None
    fieldName = None
    operator = None
    start = None
    value = None
    valuePath = None

    def __str__(self):
        return super().__str__()



from django.db import IntegrityError
from django.test import TestCase
from isc_common.auth.models.user import User
from isc_common.auth.models.usergroup import UserGroup


class Test_User(TestCase):

    def setUp(self):
        self.passwqord = "passwqord"
        self.username = "username"
        self.password = "username"
        self.email = "info@ivc-inform.ru"
        self.first_name = "first_name"
        self.last_name = "last_name"
        self.middle_name = "middle_name"
        self.group = UserGroup.objects.get(code='administrators')

    def test_1(self):
        User_item = User.objects.create_superuser(username=self.username, password=self.password, email=self.email)
        User_item = User.objects.get(pk=User_item.pk)
        self.assertTrue(User_item.has_usable_password())
        self.assertFalse(User_item.check_password('bad'))
        self.assertTrue(User_item.check_password(self.password))

        self.username += '1'
        User_item = User.objects.create_superuser(
            username=self.username,
            password=self.password,
            email=self.email,
            first_name=self.first_name,
            last_name=self.last_name,
            middle_name=self.middle_name,
        )
        User_item = User.objects.get(pk=User_item.pk)
        self.assertTrue(User_item.check_password(self.password))
        self.assertEquals(User_item.first_name, self.first_name)
        self.assertEquals(User_item.last_name, self.last_name)
        self.assertEquals(User_item.middle_name, self.middle_name)

    def test_2(self):
        with self.assertRaises(IntegrityError):
            User.objects.create_superuser(username=self.username, password=self.password, email=self.email)
            User.objects.create_superuser(username=self.username, password=self.password, email=self.email)

    def test_3(self):
        with self.assertRaises(IntegrityError):
            User.objects.create()

    def test_4(self):
        group1 = UserGroup.objects.create(code="group1")
        group2 = UserGroup.objects.create(code="group2")
        User_item = User.objects.create_user(usergroup= [group1, group2], username=self.username, password=self.password)
        User_item = User.objects.get(pk=User_item.pk)
        groups = User_item.usergroup.all()
        for group in groups:
            print(group)
        self.assertTrue(User_item.has_usable_password())
        self.assertTrue(User_item.check_password(self.password))




# Generated by Django 3.0.2 on 2020-01-22 10:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reports', '0003_auto_20200122_1029'),
    ]

    operations = [
        migrations.AlterField(
            model_name='jasper_reports',
            name='port',
            field=models.PositiveIntegerField(),
        ),
    ]



from django.urls import path

from tracker.views import messages_importance

urlpatterns = [

    path('Messages_importance/Fetch/', messages_importance.Messages_importance_Fetch),
    path('Messages_importance/Add', messages_importance.Messages_importance_Add),
    path('Messages_importance/Update', messages_importance.Messages_importance_Update),
    path('Messages_importance/Remove', messages_importance.Messages_importance_Remove),
    path('Messages_importance/Lookup/', messages_importance.Messages_importance_Lookup),
    path('Messages_importance/Info/', messages_importance.Messages_importance_Info),
    path('Messages_importance/Copy', messages_importance.Messages_importance_Copy),

]



import logging

from django.db.models import PositiveIntegerField
from django.forms import model_to_dict

from isc_common.fields.code_field import CodeStrictField
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRefHierarcy, BaseRefManager, BaseRefQuerySet

logger = logging.getLogger(__name__)


class Jasper_reportsQuerySet(BaseRefQuerySet):
    pass


def get_reports(request):
    from reports.models.jasper_reports_users import Jasper_reports_users

    request = DSRequest(request=request)
    jasper_reports = [dict(
        report=model_to_dict(item.report),
        editor_identifier=item.editor_identifier,
    ) for item in Jasper_reports_users.objects.filter(user=request.user)]
    return jasper_reports


class Jasper_reportsManager(BaseRefManager):

    @classmethod
    def getRecord(cls, record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'path': record.path,
            'description': record.description,
            'username': record.username,
            'password': record.password,
            'report_id': record.report_id,
            'host': record.host,
            'port': record.port,
            'parent': record.parent.id if record.parent else None
        }
        return res

    def get_queryset(self):
        return Jasper_reportsQuerySet(self.model, using=self._db)


class Jasper_reports(BaseRefHierarcy):
    path = CodeStrictField(default='/jasperserver/rest_v2/reports/')
    report_id = CodeStrictField(null=True, blank=False)
    username = CodeStrictField(default='printer')
    password = CodeStrictField(default='printer')
    host = CodeStrictField()
    port = PositiveIntegerField()

    objects = Jasper_reportsManager()

    def __str__(self):
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, path: {self.path}, description: {self.description}"

    class Meta:
        verbose_name = 'Отчеты'



import inspect
import logging
import os
import re
from shutil import copyfile, rmtree

from code_generator.app.icons import Icons
from code_generator.core.writer import Writer

logger = logging.getLogger(__name__)


class Tag(Writer):
    params = []
    value = []
    pos = None
    raw_tag = None
    absolute_position = None
    unquoted = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.pos = kwargs.get('pos')
        if self.pos is None:
            raise Exception(f'Не определен pos.')

        self.raw_tag = kwargs.get('raw_tag')
        if self.raw_tag is not None and self.raw_tag != '':
            if self.raw_tag is None:
                raise Exception(f'Не определен raw_tag.')

            self.code_type = kwargs.get('code_type')
            if self.code_type is None:
                raise Exception(f'Не определен code_type.')

            self.value = kwargs.get('value')
            if self.value is None:
                raise Exception(f'Не определен value.')

            if not isinstance(self.value, list):
                self.value = [self.value]

            self.value_len = max([len(str(item)) for item in self.value])

            lst = self.raw_tag.replace('${', '').replace('}', '').split(':')
            self.params = lst[1:]

            self.unquoted = self.get_param('unquoted')  # Строковый параметр не заквочивается
            self.absolute_position = self.get_param('absolute_position')  # Праметр заменяется непосредственно в месте нахождения.
            # если замена многострочная, то остальные строки подтягиваются на уровень вставки первой строки

    def get_param(self, param_name):
        res = [item for item in self.params if item == param_name]
        return len(res) > 0

    def get_str(self):
        step = 1
        new_pos = 0
        res = ''
        if self.raw_tag is not None and self.raw_tag != '':
            for _value in self.value:
                if isinstance(_value, str):
                    if not self.unquoted:
                        _value = self.dbl_qutes_str(_value)

                if isinstance(_value, bool):
                    if self.code_type == 'ts':
                        _value = self.uncapitalize(str(_value))

                if step == 1:
                    if self.absolute_position:
                        res = _value
                    else:
                        res = self.fill_space(self.pos) + _value

                    new_pos = self.pos + self.value_len
                    step += 1
                else:
                    res += self.fill_space(self.pos) + _value

            self.pos = new_pos
        return str(res)


class Model(Writer):
    image_dir_name = 'common-webapp'
    template_name = None
    output_file_name = None
    class_name = None

    name_app = None
    code_app = None
    path_app = None

    def get_model_attributes(self):
        members = [m for m in inspect.getmembers(self) if not (m[0].startswith('__') and m[0].endswith('__')) and m[1] is not None and not inspect.ismethod(m[1])]
        return members

    def write_bool(self, value):
        if self.code_type == 'ts':
            return self.uncapitalize(value)
        elif self.code_type == 'py':
            return self.capitalize(value)
        else:
            raise Exception(f'Unknowm code_type.')

    check_patern = re.compile('(.+?)(\$?\{.*?\})|(.*)')

    def check_line(self, line):
        for tag in self.check_patern.findall(line):
            if tag[1] is None or tag[1] == '':
                return True
            else:
                return False

    def check_line1(self, line):
        for tag in self.check_patern.findall(line):
            if tag[1] is None or tag[1] == '':
                return True, ''
            else:
                return False, tag[1]

    def check_releases_template(self, lines):
        messages = []

        for line in lines:
            check, tag = self.check_line1(line)
            if not check:
                if len(messages) == 0:
                    messages.append(f'В шаблоне {self.template_name} не замещены следуюжие теги:')
                messages.append(self.fill_space(4) + tag)

        if len(messages) > 0:
            raise Exception('\n'.join(messages))

    def mod_emtity(self, value, results_finded):
        pos = 0
        if isinstance(results_finded, list):
            str = ''
            for _tuple in results_finded:
                if isinstance(_tuple, tuple):
                    str += _tuple[0]
                    pos += len(_tuple[0])
                    tag = Tag(raw_tag=_tuple[1], value=value, pos=pos, code_type=self.code_type)
                    str += tag.get_str()
                    pos += tag.pos
                    str += _tuple[2]
                    pos += len(_tuple[2])
                else:
                    raise Exception(f'{results_finded} in not tuple')
            return str
        else:
            raise Exception(f'{results_finded} in not list')

    def write_entity(self, replace_map=None):
        from code_generator.app.app_codeblock import ModelCodeBlock

        template_path = f'{self.template_path}{os.sep}{self.template_name}'

        self.check_path(self.template_path)

        out_path1 = f'{self.path_output}{os.sep}{self.path_app}'
        out_path = f'{self.path_output}{os.sep}{self.path_app}{os.sep}ts{os.sep}generated'
        if os.path.exists(out_path1):
            rmtree(out_path1)
        os.makedirs(out_path)
        copyfile(self.check_path(f'{self.template_path}{os.sep}__init__.pmd'), f'{out_path1}{os.sep}__init__.py')

        out_path = f'{out_path}{os.sep}{self.output_file_name}'

        out_file = open(out_path, "w+")  # open for append
        lines = []

        for line in open(template_path):
            self.print_dict(**dict(self.get_model_attributes()))
            for key, value in self.get_model_attributes():
                if isinstance(value, bool) or isinstance(value, str) or isinstance(value, int):
                    pattern = self.get_pattern(key)
                    result = pattern.findall(line)
                    line = self.mod_emtity(value, result)
                    if line == '' or self.check_line(line):
                        break

                elif isinstance(value, ModelCodeBlock):

                    pattern = self.get_pattern(key)
                    result = pattern.findall(line)
                    line = self.mod_emtity(value.code, result)
                    if line == '' or self.check_line(line):
                        break

            if line != '':
                lines.append(line)

        self.check_releases_template(lines)
        for line in lines:
            out_file.write(f'{line}\n')
        out_file.close()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        from project.settings import STATICFILES_DIRS

        icons_path = [path for key, path in STATICFILES_DIRS if key == self.image_dir_name]
        if len(icons_path) != 1 or not os.path.exists(icons_path[0]):
            raise Exception(f'Не найден путь {icons_path}')
        self.icons = Icons(icons_path[0])

        if kwargs is not None:
            for key, value in kwargs.items():
                if isinstance(value, (list, tuple)):
                    setattr(self, key, [Model(x) if isinstance(x, dict) else x for x in value])
                else:
                    setattr(self, key, Model(key=value) if isinstance(value, dict) else value)



# Generated by Django 2.2.4 on 2019-08-21 12:25

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0011_auto_20190821_0820'),
    ]

    operations = [
        migrations.AddField(
            model_name='holidays',
            name='props',
            field=bitfield.models.BitField((('holiday', 'Праздник'), ('shorted', 'Сокращенный')), db_index=True, default=None),
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-03 19:36

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('refs', '0006_remove_type_param_values_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='type_param_values',
            name='type',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='refs.Type_param'),
        ),
    ]



# Generated by Django 2.2.5 on 2019-09-03 21:26

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('tracker', '0046_auto_20190903_1905'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='messages',
            unique_together={('guid', 'theme', 'state', 'message', 'user')},
        ),
    ]
